import { ClientConfig } from '../config/Config';
import { Cluster } from './Cluster';
import { Member } from './Member';
import { InitialMembershipListener, InitialMembershipEvent, MembershipEvent } from './MembershipListener';
/**
 * {@link LoadBalancer} allows you to send operations to one of a number of endpoints (Members).
 * It is up to the implementation to use different load balancing policies.
 * <p>
 * If Client is configured with {@link ClientNetworkConfig.smartRouting},
 * only the operations that are not key based will be routed to the endpoint
 * returned by the LoadBalancer. If it is not, the LoadBalancer will not be used.
 * <p>
 */
export interface LoadBalancer {
    /**
     * Initializes the LoadBalancer.
     *
     * @param cluster the Cluster this LoadBalancer uses to select members from.
     * @param config  the ClientConfig.
     */
    initLoadBalancer(cluster: Cluster, config: ClientConfig): void;
    /**
     * Returns the next member to route to.
     *
     * @return Returns the next member or `null` if no member is available
     */
    next(): Member | null;
    /**
     * Returns the next data member to route to.
     *
     * @return Returns the next data member or `null` if no data member is available
     * @deprecated Since 5.0, the method is unused
     */
    nextDataMember(): Member | null;
    /**
     * Returns whether this instance supports getting data members through a call to {@link nextDataMember()}.
     *
     * @return Returns `true` if this load balancer can get a data member.
     * @deprecated Since 5.0, the method is unused
     */
    canGetNextDataMember(): boolean;
}
/**
 * Abstract Load Balancer to be used in built-in and user-provided
 * {@link LoadBalancer} implementations.
 */
export declare abstract class AbstractLoadBalancer implements LoadBalancer, InitialMembershipListener {
    private members;
    private dataMembers;
    private cluster;
    abstract next(): Member | null;
    abstract nextDataMember(): Member | null;
    abstract canGetNextDataMember(): boolean;
    initLoadBalancer(cluster: Cluster, config: ClientConfig): void;
    init(event: InitialMembershipEvent): void;
    memberAdded(membership: MembershipEvent): void;
    memberRemoved(membership: MembershipEvent): void;
    protected getDataMembers(): Member[];
    protected getMembers(): Member[];
    private setMembers;
}
